// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
//#include "bhv_shoot2008.h"
#include <rcsc/action/bhv_shoot2008.h>
#include "role_center_forward.h"
#include "bhv_keep_shoot_chance.h"
#include "kick_table.h"
#include "strategy.h"

#include "bhv_basic_offensive_kick.h"
#include "bhv_basic_move.h"


#include <rcsc/formation/formation.h>

#include <rcsc/player/player_agent.h>
#include <rcsc/player/intercept_table.h>
#include <rcsc/player/debug_client.h>

#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>
#include <rcsc/common/say_message_parser.h>
#include <rcsc/common/audio_memory.h>

#include <rcsc/action/basic_actions.h>
#include <rcsc/action/body_go_to_point.h>
#include <rcsc/action/body_kick_one_step.h>
#include <rcsc/action/body_intercept.h>
#include <rcsc/action/neck_scan_field.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>
#include <rcsc/action/neck_turn_to_goalie_or_scan.h>
#include <rcsc/action/neck_turn_to_low_conf_teammate.h>
#include <rcsc/action/bhv_go_to_point_look_ball.h>
#include <rcsc/action/view_wide.h>

#include <rcsc/geom/sector_2d.h>
#include <rcsc/geom/rect_2d.h>
#include <rcsc/math_util.h>

#include "neck_default_intercept_neck.h"
#include "neck_offensive_intercept_neck.h"

/*-------------------------------------------------------------------*/
/*!

*/
void
RoleCenterForward::execute( rcsc::PlayerAgent * agent )
{
    bool kickable = agent->world().self().isKickable();
    if ( agent->world().existKickableTeammate()
         && agent->world().teammatesFromBall().front()->distFromBall()
         < agent->world().ball().distFromSelf() )
    {
        kickable = false;
    }

    if ( kickable )
    {
        doKick( agent );
    }
    else
    {
        doMove( agent );
    }

}

/*-------------------------------------------------------------------*/
/*!

*/
void
RoleCenterForward::doKick( rcsc::PlayerAgent * agent )
{
    switch ( Strategy::get_ball_area( agent->world().ball().pos() ) ) {
    case Strategy::BA_CrossBlock:
    case Strategy::BA_Stopper:
    case Strategy::BA_Danger:
    case Strategy::BA_DribbleBlock:
    case Strategy::BA_DefMidField:
    case Strategy::BA_DribbleAttack:
    case Strategy::BA_OffMidField:
    case Strategy::BA_Cross:
    case Strategy::BA_ShootChance:
    default:
	
        Bhv_BasicOffensiveKick().execute( agent );
        break;
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
void
RoleCenterForward::doMove( rcsc::PlayerAgent * agent )
{
    const rcsc::WorldModel & wm = agent->world();

    int ball_reach_step = 0;
    if ( ! wm.existKickableTeammate()
         && ! wm.existKickableOpponent() )
    {
        ball_reach_step
            = std::min( wm.interceptTable()->teammateReachCycle(),
                        wm.interceptTable()->opponentReachCycle() );
    }
    const rcsc::Vector2D base_pos
        = wm.ball().inertiaPoint( ball_reach_step );

    rcsc::Vector2D home_pos
        = formation().getPosition( agent->config().playerNumber(),
                                   base_pos );
    if ( rcsc::ServerParam::i().useOffside() )
    {
        home_pos.x = std::min( home_pos.x, wm.offsideLineX() - 1.0 );
    }

    rcsc::dlog.addText( rcsc::Logger::ROLE,
                        "%s: HOME POSITION=(%.2f, %.2f) base_point(%.1f %.1f)"
                        ,__FILE__,
                        home_pos.x, home_pos.y,
                        base_pos.x, base_pos.y );
 	
	rcsc::Vector2D ball_pos = wm.ball().pos();
	rcsc::Vector2D get_pass_pos(wm.ball().pos().x + 3 , ball_pos.y * 0.75);
	//struct passmessage {int,int,rcsc::Vector2D};
	


    switch ( Strategy::get_ball_area( base_pos ) ) {
    case Strategy::BA_CrossBlock:
    case Strategy::BA_Stopper:
    case Strategy::BA_Danger:
    case Strategy::BA_DribbleBlock:
    case Strategy::BA_DefMidField:
    case Strategy::BA_DribbleAttack:
    case Strategy::BA_OffMidField:
    case Strategy::BA_Cross:
    case Strategy::BA_ShootChance:
    default:
      {
	if (abs(agent->world().audioMemory().passTime().cycle() - agent->world().time().cycle()) < 2 ){
	  if (agent->world().audioMemory().pass().front().receiver_ == 11 ){
	    Bhv_BasicMove( home_pos ).execute( agent );
	    std::cout << "heard!" << std::endl;
	    rcsc::Vector2D target_point = agent->world().audioMemory().pass().front().receive_pos_;
	    std::cout << target_point.x 
		      << ","
		      << target_point.y
		      << std::endl;
	    rcsc::dlog.addText (rcsc::Logger::SENSOR, __FILE__": Heard X:%f Y:%f", target_point.x, target_point.y);
	    rcsc::Body_GoToPoint ( target_point,
				   5.0,
				   100,
				   5,
				   false,
				   false,
				   30.0
				   );
	    break;
	  }
	}
	
        Bhv_BasicMove( home_pos ).execute( agent );
        break;
      }
    }
}
